diff --git a/src/backend/parser/parse_relation.c b/src/backend/parser/parse_relation.c
new file mode 100644
index 864ea9b..0afe177
--- a/src/backend/parser/parse_relation.c
+++ b/src/backend/parser/parse_relation.c
@@ -395,6 +395,20 @@ searchRangeTableForRel(ParseState *pstat
 		{
 			RangeTblEntry *rte = (RangeTblEntry *) lfirst(l);
 
+			/*
+			 * Ignore RTEs that do not automatically take their names from
+			 * database objects, and have not been supplied with aliases
+			 * (e.g., unnamed joins).  Such RTEs cannot possibly match the
+			 * RangeVar, and cannot be referenced from anywhere in the query.
+			 * Thus, it would be misleading to complain that they can't be
+			 * referenced from particular parts of the query.
+			 */
+			if (!rte->alias &&
+				(rte->rtekind == RTE_SUBQUERY ||
+				 rte->rtekind == RTE_JOIN ||
+				 rte->rtekind == RTE_VALUES))
+				continue;
+
 			if (rte->rtekind == RTE_RELATION &&
 				OidIsValid(relId) &&
 				rte->relid == relId)
diff --git a/src/test/regress/expected/join.out b/src/test/regress/expected/join.out
new file mode 100644
index 9b8638f..2bbf3bf
--- a/src/test/regress/expected/join.out
+++ b/src/test/regress/expected/join.out
@@ -2895,6 +2895,26 @@ select bar.*, unnamed_join.* from
 ERROR:  FOR UPDATE cannot be applied to a join
 LINE 3:   for update of bar;
                         ^
+-- Test error reporting of references to internal aliases
+select unnamed_join.* from
+  t1 join t2 on (t1.a = t2.a);
+ERROR:  missing FROM-clause entry for table "unnamed_join"
+LINE 1: select unnamed_join.* from
+               ^
+select unnamed_join.* from
+  (select * from t1 join t2 using (a)) ss join t3 on (t3.x = ss.a);
+ERROR:  missing FROM-clause entry for table "unnamed_join"
+LINE 1: select unnamed_join.* from
+               ^
+select unnamed_subquery.* from
+  (select * from t1);
+ERROR:  missing FROM-clause entry for table "unnamed_subquery"
+LINE 1: select unnamed_subquery.* from
+               ^
+insert into t1 values (1, 10), (2, 20) returning "*VALUES*".*;
+ERROR:  missing FROM-clause entry for table "*VALUES*"
+LINE 1: insert into t1 values (1, 10), (2, 20) returning "*VALUES*"....
+                                                         ^
 --
 -- regression test for 8.1 merge right join bug
 --
diff --git a/src/test/regress/sql/join.sql b/src/test/regress/sql/join.sql
new file mode 100644
index 3e5032b..4d4ebc3
--- a/src/test/regress/sql/join.sql
+++ b/src/test/regress/sql/join.sql
@@ -669,6 +669,19 @@ select bar.*, unnamed_join.* from
   (t1 join t2 using (a) as foo) as bar, t3 as unnamed_join
   for update of bar;
 
+-- Test error reporting of references to internal aliases
+
+select unnamed_join.* from
+  t1 join t2 on (t1.a = t2.a);
+
+select unnamed_join.* from
+  (select * from t1 join t2 using (a)) ss join t3 on (t3.x = ss.a);
+
+select unnamed_subquery.* from
+  (select * from t1);
+
+insert into t1 values (1, 10), (2, 20) returning "*VALUES*".*;
+
 --
 -- regression test for 8.1 merge right join bug
 --
